// SPDX-FileCopyrightText: Deutsches Elektronen-Synchrotron DESY, MSK, http://msk.desy.de
// SPDX-License-Identifier: LGPL-3.0-or-later
#pragma once

#include <map>
#include <string>
/** The codes are defined in the MeCom Protocol Specification by meerstetter, document ID 5117C Section 1.3.4
 *  "Server Error Codes" on page 4. */
enum class TecErrorCode {
  NO_ERROR = 0,
  EER_CMD_NOT_AVAILABLE = 1,
  EER_DEVICE_BUSY = 2,
  ERR_GENERAL_COM = 3,
  EER_FORMAT = 4,
  EER_PAR_NOT_AVAILABLE = 5,
  EER_PAR_NOT_WRITABLE = 6,
  EER_PAR_OUT_OF_RANGE = 7,
  EER_PAR_INST_NOT_AVAILABLE = 8,
  ERR_PAR_GENERAL_FAILURE = 9
};

static std::map<TecErrorCode, std::string> TecErrorDescriptions = {
    {TecErrorCode::NO_ERROR, ""},
    {TecErrorCode::EER_CMD_NOT_AVAILABLE, "Command not available."},
    {TecErrorCode::EER_DEVICE_BUSY, "Device is busy."},
    {TecErrorCode::ERR_GENERAL_COM, "General communication error."},
    {TecErrorCode::EER_FORMAT, "Format error."},
    {TecErrorCode::EER_PAR_NOT_AVAILABLE, "Parameter is not available."},
    {TecErrorCode::EER_PAR_NOT_WRITABLE, "Parameter is read only."},
    {TecErrorCode::EER_PAR_OUT_OF_RANGE, "Value is out of range."},
    {TecErrorCode::EER_PAR_INST_NOT_AVAILABLE, "Instance is not available."},
    {TecErrorCode::ERR_PAR_GENERAL_FAILURE, "Parameter general Error. Device internal failure on this parameter."},
};
